(declare (unit all-music))

(include "logger.sci")

#>
#include <curses.h>
#include <cdk.h>
#include <xmmsclient/xmmsclient.h>
<#

(define filter-string "")

; Show all music in the library
(define (show-all-music)
  (logger "In show-all-music.\n")
  (let* (
	 [null1 (logger "About to xmmsc_coll_universe.\n")]
	 ; Get a collection reference
	 [collref (xmmsc_coll_universe)]
	 ; Get the ids from the collection
	 [result (my-xmms-wait (xmmsc_coll_query_ids conn collref #f 0 0))]
	 [null1 (logger "Result: " result nl)]
	 [colls (result->int-list result)])

    (logger "colls; media ids: " colls nl)

    (show-media colls highlight-current-playlist-good)

    (show-all-music-help)

    (logger "show-media colls done\n")

    ; Cleanup
    (xmmsc_result_unref result)

    (logger "Done cleanup.\n")))

; Show all music with a filter (which we prompt for)
(define (filter-all-music)
  (logger "In filter-all-music.\n")
  (let* (
	 [new-filter-string (my-getString "Enter Filter" "Filter: " filter-string)])
    (set! filter-string new-filter-string)
    (display-filtered-all-music)))

(define current-filter #f)

; Actual display is seperate so that we can refresh the screen
; without too much trouble.
(define (display-filtered-all-music)
  (set! current-filter #f)
  (define-external temp_col c-pointer)
  (let* (
	 [parse-ok? (and filter-string
			 (eq? 1 (xmmsc_coll_parse filter-string (location temp_col))))])
    (logger "Parse-ok?: " parse-ok? nl)
    ; Only continue if the parse succeeded.
    (cond
      [parse-ok?
	(let* (
	       [result (my-xmms-wait (xmmsc_coll_query_ids conn temp_col #f 0 0))]
	       [null2 (logger "Result: " result nl)]
	       [colls (result->int-list result)])

	  (logger "colls; media ids: " colls nl)

	  ; Don't try to show the empty list
	  (cond
	    [(eq? colls '())
	     (popupLabel cdk_screen (make_strings (list "No results found.")) 1)]
	    [else
	      ; Let other filter bits know it worked out
	      (set! current-filter temp_col)

	      (show-media colls highlight-current-playlist-good)

	      (show-filter-help)

	      ; Set a new refresh function
	      (global-set! 'refresh-current (refresh-with display-filtered-all-music))

	      ; Set the system state
	      (global-set! 'system-state 'filter)

	      (logger "show-media colls done\n")

	      ; Cleanup
	      (xmmsc_result_unref result)

	      (logger "Done cleanup.\n")]))]
      [else
	(popupLabel cdk_screen (make_strings (list "Filter parse failed!")) 1)])))

; Create a new collection based on the current filter value
(define (filter-new-collection)
  (cond
    ; Only try to save the filter if it's been approved
    [current-filter
      (let* (
	     [name (my-getString "Enter New Collection Name" "Name: " "")])
	(xmmsc_coll_save
	  conn
	  current-filter
	  name
	  (foreign-value "XMMS_COLLECTION_NS_COLLECTIONS" c-string)))]))
