; This is Cluck version 1.0

(use lolevel)
(use srfi-1)
(use utils)
(use srfi-18)
; These are put in by the compiler, depending on whether we're
; statically linking or not.
;(use xmms2client)
;(use fmt)
;(use cdk)

(declare (uses clucklib))
(declare (uses playlists))
(declare (uses collections))
(declare (uses bindings))
(declare (uses menu))
(declare (uses all-music))
(declare (uses status))
(declare (uses cdk))
(declare (uses columns))
(declare (uses props))
(declare (uses tags))
(declare (uses config))
(declare (uses directory))
(declare (uses ratings))

(include "logger.sci")

#>
#include <curses.h>
#include <cdk.h>
#include <xmmsclient/xmmsclient.h>
<#

; Callback functions must be defined in the file they are used.
; *sigh*  At least they can be stubs.
;(include "callbacks.scm")

;*************
; Global defines, mostly used by callback functions
;*************
;
; Stores information about the current elements in the selection
; list, so the callbacks and such can understand what it is they are
; looking at.
(define selection-items #f)
; Store information about the state of the universe so we don't have
; to keep binding various keys; that seems to make CDK upset.
(define system-state 'current-playlist)
; The name of the currently-being-viewed collection
(define current-collection #f)
; The name of the currently-being-viewed playlist
(define current-playlist #f)
; Whether we're viewing a collection or playlist at this exact
; moment.
(define collection-type #f)
; Are we writing props back to the files automatically?
(define prop-writeback #f)
; Auxillary function for refresh-with
(define (refresh-with helper) (lambda () (helper) (reset-focus) (drawCDKSelection selection-widget 1)))
; A function that can always be used to refresh the current display.
(define refresh-current (refresh-with show-menu))
; Starting directory in the directory view
(define starting-directory #f)
; Extra bits to add to the status
(define status-extra #f)
; If set, reload the props, albums, tags, etc.
(define reload-props #f)
; The list of all known properties, albums
(define all-props '())
(define all-artists '())
; The list of all known tags, where tags are defined as integer
; properties that we've only ever seen at 0 or 1
(define all-tags '())
;(define all-albums '())
; What columns to show, and in which order.
(define music-columns
  '(("ID" "id")
    ("Artist" "artist")
    ("Album" "album")
    ("Title" "title")
    ("Dur" "duration")
    ("URL" "url")))
; What columns to sort on, and in which order
(define sort-columns
  '(("Artist" "artist")
    ("Album" "album")
    ("Title" "title")))
; Where the focus was in various places
(define focus-alist '())
; The current playlist in the playlists view
(define playlist-name #f)
; The name of the currently-being-viewed collection name space
(define current-collection-ns (foreign-value "XMMS_COLLECTION_NS_COLLECTIONS" c-string))
; The name of the file to log to; gets blown away on each new run.
(define log-file (conc "/tmp/cluck.log." (current-process-id)))
; Log or not?
(define logging #f)
; The user config directory
(define config-dir #f)
; Things to save to the config file
(define config-saves (list 'music-columns
			   'sort-columns
			   'all-props
			   'all-tags
			   'all-artists
			   'starting-directory
			   'userid
			   'auto-rating-change
			   'auto-rating
			   'auto-skip
			   'auto-skip-limit
			   'little-help
			   'prop-writeback))
; Userid to use for ratings
(define userid #f)
; The number to add or subtract for a positive or negative rating
; event; defaults to take 5 events to change a level.
(define auto-rating-change #f)
; Whether to auto-rate
(define auto-rating #f)
; Is auto-skip on?
(define auto-skip #f)
; What's the limit below which we skip?
(define auto-skip-limit 2)
; Formats for highlighting special cases
(define bad-highlight "</58>")
(define good-highlight "</B></24>")
(define other1-highlight "</B></56>")
(define other2-highlight "</21>")
; What to print when no data is found
(define no-data "---")
; State reversion variables
(define previous-state #f)
(define refresh-previous #f)
; Is the bottom pane on?
(define little-help #t)
; The directory the current song is in; set by the status bits
(define current-song-directory #f)
; Forward encoding for files; the reverse is calculated by flipping
; these.  To use you *MUST* run regexp-escape on whichever side
; you're going to use as the "from" side (i.e. the left/first side)
(define xmms-file-name-encoding
  ; Space gets encoded as +
  '((" " . "+")
    ; Various things get encoded to 3 character patterns
    ("'" . "%27")
    ("&" . "%26")
    ("[" . "%5b")
    ("]" . "%5d")
    ("!" . "%21")
    ("#" . "%23")
    (";" . "%3b")
    ("," . "%2c")
    (")" . "%29")
    ("(" . "%28")))
; quit when this says 1
(define quit 0)

; Deal with command line arguments
(for-each
  (lambda (arg)
    (cond
      [(or
	 (string= arg "--help")
	 (string= arg "-h"))
       (fmt #t "

Options:

-h/--help	This message.

-d		Turn on logging to /tmp/cluck.log.[pid]

-r		Reload tag/property data

")
	(exit 0)]
       [(string= arg "-r")
	(set! reload-props #t)]
       [(string= arg "-d")
	(set! logging #t)]
	))
  (argv))

; Open the log port

(cond
  [logging
    ; The port for logging; also where we clear the file
    (define log-port (begin (delete-file* log-file) (open-output-file log-file)))])

; This has to be global so callbacks can see it.
(define conn (xmmsc_init "cluck"))

; Open our connection to the server
(xmmsc_connect conn (getenv "XMMS_PATH"))

; Read config data
(set! config-dir (xmmsc_userconfdir_get_with_leak))
(logger "config dir is: " config-dir nl)
(read-config)

; Load the global data if it's not already set
(cond
  [(or
     (not all-props)
     (eq? all-props '())
     reload-props)
   (fmt #t "

Collecting information about all media; this is slow, and it's not
my fault.  libxmmsclient needs statistical information retrieval.

On the plus side, it should only happen once.  If you want to
re-load the information, run cluck with '-r' as an option.

")

	(get-all-props)])
(logger "all-props: " all-props nl)
(logger "all-artists: " all-artists nl)
(logger "all-tags: " all-tags nl)
;(logger "all-albums: " all-albums nl)


; *************************
; Now we try CDK bits.
; ******


(logger "make window.\n")
(define-external curses_window c-pointer (initscr))
(logger "make screen.\n")
(define cdk_screen (initCDKScreen curses_window))

(logger "init color.\n")
(initCDKColor)

; Global so callbacks can access it.
(define maxwidth (- (foreign-value "getmaxx((WINDOW *)curses_window)" int) 2))


(define-external blank_ext c-pointer
		 (make_strings (list
				 (fmt #f (fit maxwidth ""))
				 (fmt #f (fit maxwidth ""))
				 (fmt #f (fit maxwidth ""))
				 (fmt #f (fit maxwidth "")))))

; Initial setup for the status widget
(define status-widget
  (newCDKLabel
    cdk_screen
    (foreign-value "CENTER" unsigned-int)
    (foreign-value "TOP" unsigned-int)
    blank_ext
    4
    1
    0))
(drawCDKLabel status-widget 1)

; Initial setup for the help section
(define help-widget #f)

(define-external choices c-pointer (make_strings (list "   " "-->")))

; Global so callbacks can access it
(define selection-widget #f)
(define remake-selection-widget #f)

; The height of the widget gets reset based on the presence of the
; bottom pane
(define (make-selection-widget)
  (logger "In make-selection-widget; little-help: " little-help nl)
  (global-set! 'remake-selection-widget #t)

  ; Clear out the old widget, if any
  (cond
    [selection-widget
      (eraseCDKSelection selection-widget)
      (setCDKSelectionPreProcess selection-widget #f #f)
      (injectCDKSelection selection-widget (foreign-value "KEY_ENTER" unsigned-int))
      (setCDKObjectWaiting selection-widget #f #f)
      ])

  ; Make the selection widget
  (global-set! 'selection-widget
	       (newCDKSelection
		 cdk_screen
		 (foreign-value "CENTER" unsigned-int)
		 5
		 (foreign-value "RIGHT" unsigned-int)
		 (cond
		   [little-help -10]
		   [else -5])
		 0
		 (foreign-value "NULL" c-pointer)
		 (foreign-value "NULL" c-pointer)
		 0
		 choices
		 2
		 (foreign-value "A_REVERSE" unsigned-int)
		 1
		 0))

  (drawCDKSelection selection-widget 1)

  ; Now make the help widget, if any
  (cond
    [help-widget
      (eraseCDKSelection help-widget)
      (destroyCDKSelection help-widget)])

  (cond
    [little-help
      (global-set! 'help-widget
		   (newCDKLabel
		     cdk_screen
		     (foreign-value "CENTER" unsigned-int)
		     (foreign-value "BOTTOM" unsigned-int)
		     blank_ext
		     4
		     1
		     0))
      (drawCDKLabel help-widget 1)]
    [else (global-set! 'help-widget #f)])

  ; Bind the bindings
  (logger "Setting bindings.\n")
  (set-bindings)

  ; Put the status in play
  (status-setup)

  (drawCDKLabel status-widget 1)

  (cond
    [refresh-current (refresh-current)]))

(make-selection-widget)

(logger "\n\n\n")
(logger "list: " selection-widget nl)

(logger "Showing menu.\n")
(show-menu)
(logger "Done showing menu.\n")

(define (selection-loop choice)
  (logger "choice: " choice ", " quit nl)
  (cond
    [(and
       remake-selection-widget
       (not (= quit 1)))
     (global-set! 'remake-selection-widget #f)
     (selection-loop (activateCDKSelection selection-widget #f))]))

(logger "Starting selection-loop.\n")

(selection-loop (activateCDKSelection selection-widget #f))

(logger "Done selection-loop.\n")

(logger "In cleanup.\n")
; Clean up.
(destroyCDKSelection selection-widget)
(destroyCDKScreen cdk_screen)
(endCDK)

; Write out config data
(write-config)

(xmmsc_unref conn)
