(declare (unit columns))

(include "logger.sci")

#>
#include <curses.h>
#include <cdk.h>
#include <xmmsclient/xmmsclient.h>
<#

; alist to convert raw columns names that we've actually heard of
; into more complete descriptions
(define column-descs #f)

(define write-columns #f)
(define read-columns #f)

; Go from wherever to the main menu
(define (configure-columns to-write to-read)
  ; For some reason, we can't see userid during initial startup, so
  ; we put the list setup here.
  (set! column-descs
    (list '("id" "ID")
	  '("artist" "Artist")
	  '("title" "Title")
	  '("album" "Album")
	  '("duration" "Dur")
	  '("url" "URL")
	  '("comment" "Comment")
	  '("date" "Date")
	  '("bitrate" "Bit Rate")
	  '("genre" "Genre")
	  (list (conc "rating." userid) "Score")
	  ))
  (logger "In configure-columns, descs: " column-descs nl)
  (set! write-columns to-write)
  (set! read-columns (to-read))
  (let*
    ; Get all the props that aren't already in use.
    ([unused (lset-difference
	       string=
	       (map first all-props)
	       (map second read-columns))])
    (logger "In configure-columns: unused: " unused nl)
    (show-columns-help)
    ; The list here is the current columns, in order, followed
    ; by the rest.
    (set-selection
      (fmt #f (fit maxwidth "Configure Columns"))
      ; The scheme selection list (i.e. the raw data)
      (append
	(map second read-columns)
	unused)
      ; The actual text to display.
      (append
	(map
	  (lambda (item)
	    (conc good-highlight (first item)))
	  read-columns)
	unused))))

; Add the current columns to the list
(define (columns-add object)
  (logger "In columns-add.\n")
  (handle-choices
    object
    #t
    ; Turn the list of choices into a list of columns
    (lambda (items)
      (map
	(lambda (x)
	  (list-ref selection-items x))
	items))
    ; For each id (now a raw column name string), add it to
    ; read-columns at the end.
    (lambda (column)
      (logger "In columns-add: " column nl)
      (cond
	; Only add it if it's not already there
	[(not (member column (map second read-columns)))
	 (global-set!
	   write-columns
	   (append
	     read-columns
	     ; Put together human-readable and raw; double-list due
	     ; to the behaviour of append
	     (list
	       (list
		 ; Get the second (i.e. human-readable) value from
		 ; column-descs, or from a synthetic list with just
		 ; the raw value in it.
		 (second
		   (or
		     (assoc column column-descs)
		     (list column column)))
		 column))))])
      (logger "In columns-add: read-columns: " read-columns nl))))

; Remove the current columns from the list
(define (columns-remove object)
  (logger "In columns-remove.\n")
  (handle-choices
    object
    #t
    ; Turn the list of choices into a list of columns
    (lambda (items)
      (map
	(lambda (x)
	  (list-ref selection-items x))
	items))
    ; For each id (now a raw column name string), remove it from
    ; read-columns
    (lambda (column)
      (logger "In columns-remove: " column nl)
      (global-set!
	write-columns
	(remove (lambda (x) (string= (second x) column)) read-columns)))))

; Move the selected entry up the columns list
(define (columns-up object)
  (logger "In columns-up\n")
  ; If we're already at the top, don't try it!
  (cond
    [(not (= 0 (getCDKSelectionCurrent object)))
     ; Do the actual movement: walk the list of indices, and at the
     ; pos-1 index put the pos item, and at the pos index put the
     ; pos-1 item
     (global-set!
       write-columns
       (map
	 (lambda (index)
	   (cond
	     [(= index (getCDKSelectionCurrent object))
	      (list-ref read-columns (- index 1))]
	     [(= index (- (getCDKSelectionCurrent object) 1))
	      (list-ref read-columns (+ index 1))]
	     [else (list-ref read-columns index)]))
	 (iota (length read-columns))))
     ; Move the focus to follow
     (global-set!
       'focus-alist
       (alist-cons
	 "columns"
	 (- (getCDKSelectionCurrent object) 1)
	 (alist-delete
	   "columns"
	   focus-alist)))
     (refresh-current)]))

; Move the selected entry down the columns list
(define (columns-down object)
  (logger "In columns-down: " (getCDKSelectionCurrent object) ", " (length read-columns) "\n")
  ; If we're already at the bottom, don't try it!
  (cond
    [(not (<= (- (length read-columns) 1) (getCDKSelectionCurrent object)))
     ; Do the actual movement: walk the list of indices, and at the
     ; pos index put the pos+1 item, and at the pos+1 index put the
     ; pos-1 item
     (global-set!
       write-columns
       (map
	 (lambda (index)
	   (cond
	     [(= index (getCDKSelectionCurrent object))
	      (list-ref read-columns (+ index 1))]
	     [(= index (+ (getCDKSelectionCurrent object) 1))
	      (list-ref read-columns (- index 1))]
	     [else (list-ref read-columns index)]))
	 (iota (length read-columns))))

     ; Move the focus to follow
     (global-set!
       'focus-alist
       (alist-cons
	 "columns"
	 (+ (getCDKSelectionCurrent object) 1)
	 (alist-delete
	   "columns"
	   focus-alist)))
     (refresh-current)]))
