(declare (unit playlists))

(include "logger.sci")

#>
#include <curses.h>
#include <cdk.h>
#include <xmmsclient/xmmsclient.h>
<#

; The function that takes us from some other view to the current
; playlist view.
;
; Refresh the given widget (which must be a selection list!) with
; the current playlist.
(define (show-current-playlist)
  (show-current-playlist-help)
  (logger "In show-current-playlist.\n")
  (show-media (playlist-get-ids) playlist-highlight #t))

; Highlight the currently playing song only as good,  and only if
; we're playing or paused.  Also highlight any songs that will be
; auto-skipped as bad.
(define (playlist-highlight id info current-playlist-ids currently-playing-id)
  (let*
    ([real-status (status-get-playback-status)]
     [current-rating-raw (first (get-mediainfo id (list (conc "rating." userid))))]
     [current-rating
       (cond
	 [(string= no-data current-rating-raw)
	  300]
	 [else (string->number current-rating-raw)])])
    (cond
      [(and
	 (= id currently-playing-id)
	 (not (= 0 real-status)))
       (conc good-highlight info)]
      [(and
	 auto-skip
	 (< current-rating auto-skip-limit))
       (conc bad-highlight info)]
      [else info])))

; Define the function that gets activated when the "p" key is
; pressed in playlist view: play a song.
(define (playlist-play object)
  (let*
    ([real-id (playlist-current-id)])
    ; To play a song, we kind of need to be, you know, *playing*
    (my-xmms-wait (xmmsc_playback_start conn))
    (define-external obj2 c-pointer object)
    (let*
      ([list-id (getCDKSelectionCurrent object)]
       [id (list-ref selection-items list-id)]
       [null (logger "In playlist-play, list-id, id, ids: " list-id ", " id ", " (playlist-get-ids))]
       ; Given the id, we find its position in the current playlist
       [position1 (list-index (lambda (x) (eq? x id)) (playlist-get-ids))]
       ; It might not be *in* the current playlist, in which case we
       ; need to add it
       [position
	 (cond
	   [position1 position1]
	   [else
	     (playlist-add object)
	     (list-index (lambda (x) (eq? x id)) (playlist-get-ids))])]
       [null1 (logger nl nl nl "play event: " (wrt id) nl)]
       [result1 (my-xmms-wait (xmmsc_playlist_set_next conn position))]
       [result2 (my-xmms-wait (xmmsc_playback_tickle conn))])
      (cond
	[auto-rating
	  ; Wait for update *if* we're actually playing a different
	  ; song
	  (cond [(not (= id real-id)) (playlist-id-update-wait real-id)])
	  (ratings-up id)
	  (xmmsc_result_unref result1)
	  (xmmsc_result_unref result2)])
      ; Just in case we haven't updated yet
      (cond [(not (= id real-id)) (playlist-id-update-wait real-id)])
      (refresh-current))))

; Remove the selected song from the active playlist
(define (playlist-remove object)
  (logger "In playlist-rem: " object ", " system-state "\n")
  (handle-choices
    object
    #t
    ; We have to sort the items in reverse numerical order,
    ; otherwise when given (5 10), we'll remove item 5, then item 10
    ; won't be item 10 anymore.
    (lambda (items)
      (sort items >))
    (lambda (id)
      (logger "In playlist-rem removing playlist id: " id "\n")
      (my-xmms-wait-toss (xmmsc_playlist_remove_entry conn (foreign-value "XMMS_ACTIVE_PLAYLIST" c-string) id)))))

; Add the selected song or collection to the active playlist
(define (playlist-add object)
  (logger "In playlist-add: " object ", " system-state "\n")
  (handle-choices
    object
    #t
    ; Turn the list of choices into a list of song ids
    (lambda (items)
      (map
	(lambda (x)
	  (list-ref selection-items x))
	items))
    ; For each selection, add it to the current playlist.
    (lambda (id)
      (logger nl nl nl "add event: " (wrt id) nl)
      ; Here we pick between adding a whole collection, or just one
      ; item.
      (begin
	(logger "playlist-add states: " system-state ", " collection-type ", " current-playlist ", " current-collection ", " current-collection-ns nl)
	(cond
	  ; In the list of collections or playlists?  Add the whole thing
	  [(or
	     (eq? system-state 'playlists)
	     (and
	       (eq? system-state 'collections)
	       ; See if we're in a type that has a current item set
	       (or
		 (and
		   (eq? collection-type 'playlist)
		   (eq? current-playlist #f))
		 (and
		   (eq? collection-type 'collection)
		   (eq? current-collection #f)))))
	   (logger "Loading whole collection.\n")
	   (let*
	     (
	      ; Get a collection reference
	      [collref (xmmsc_coll_new (foreign-value "XMMS_COLLECTION_TYPE_REFERENCE" int))]
	      ; Define the name and namespace for the collection ref
	      [null4 (xmmsc_coll_attribute_set collref "reference" id)]
	      [null5 (xmmsc_coll_attribute_set collref "namespace" current-collection-ns)])
	     (my-xmms-wait-toss (xmmsc_playlist_add_collection conn (foreign-value "XMMS_ACTIVE_PLAYLIST" c-string) collref #f)))]
	  ; In the playlists-as-playlists list: add the
	  ; playlist by iterating over the songs, so the order
	  ; is correct.
	  [(eq? system-state 'playlists)
	   (logger "Adding single songs.\n")
	   (logger "Adding songid " id nl)
	   (my-xmms-wait-toss (xmmsc_playlist_add_id conn (foreign-value "XMMS_ACTIVE_PLAYLIST" c-string) id))]
	  [else
	    (logger "Loading song.\n")
	    (my-xmms-wait-toss (xmmsc_playlist_add_id conn (foreign-value "XMMS_ACTIVE_PLAYLIST" c-string) id))]))
      (logger "Addition completed.\n"))))

; Move the selected entry up the playlist
(define (playlist-up object)
  (logger "In playlist-up\n")
  ; If we're already at the top, don't try it!
  (cond
    [(not (= 0 (getCDKSelectionCurrent object)))
     (xmmsc_playlist_move_entry
       conn
       (foreign-value "XMMS_ACTIVE_PLAYLIST" c-string)
       (getCDKSelectionCurrent object)
       (- (getCDKSelectionCurrent object) 1))
     ; Move the focus to follow
     (global-set!
       'focus-alist
       (alist-cons
	 "current-playlist"
	 (- (getCDKSelectionCurrent object) 1)
	 (alist-delete
	   "current-playlist"
	   focus-alist)))
     (refresh-current)]))

; Move the selected entry down the playlist
(define (playlist-down object)
  (logger "In playlist-down\n")
  ; If we're already at the bottom, don't try it!
  (cond
    [(not (= (length selection-items) (getCDKSelectionCurrent object)))
     (xmmsc_playlist_move_entry
       conn
       (foreign-value "XMMS_ACTIVE_PLAYLIST" c-string)
       (getCDKSelectionCurrent object)
       (+ (getCDKSelectionCurrent object) 1))
     ; Move the focus to follow
     (global-set!
       'focus-alist
       (alist-cons
	 "current-playlist"
	 (+ (getCDKSelectionCurrent object) 1)
	 (alist-delete
	   "current-playlist"
	   focus-alist)))
     (refresh-current)]))

; Show the list of playlists, or a single playlist
(define (show-playlists)
  (cond
    ; Are we showing a single playlist?
    [playlist-name
      (show-playlist)]
    ; No.
    [else (show-playlists-list)]))

; Set the playlist to display, and show it
(define (display-playlist object)
  (cond
    ; Did we get an object?
    [object
      (define-external obj2 c-pointer object)
      (let*
	([playlist
	   (list-ref
	     selection-items
	     (getCDKSelectionCurrent object))])
	(global-set! 'playlist-name playlist))]
    ; No?  Then we're backing out
    [else 
      (global-set! 'playlist-name #f)])
  (show-playlists))

; Show whatever the currently chosen playlist is
(define (show-playlist)
  (let* (
	 ; Get a collection reference
	 [collref (xmmsc_coll_new (foreign-value "XMMS_COLLECTION_TYPE_REFERENCE" int))]
	 [null3 (logger "Loading a particular collection; " collref nl)]
	 ; Define the name and namespace for the collection ref
	 [null4 (xmmsc_coll_attribute_set collref "reference" playlist-name)]
	 [null5 (xmmsc_coll_attribute_set collref "namespace" (foreign-value "XMMS_COLLECTION_NS_PLAYLISTS" c-string))]
	 ; Get the ids from the collection
	 [ids (result->int-list (xmmsc_coll_query_ids conn collref #f 0 0))]
	 [active (result->string (xmmsc_playlist_current_active conn))])
    (logger "show-playlist ids: " ids nl)
    (logger "show-playlist active: " active nl)

    ; Turn off highlighting if we're showing the active playlist
    (cond
      [(string= active playlist-name)
       (show-media ids playlist-highlight #t)]
      [else
	(show-media ids highlight-current-playlist-good #t)])

    (global-set! 'status-extra (conc "Playlist: " playlist-name))))

; Show a list of the playlists
(define (show-playlists-list)
  (let* ([result (my-xmms-wait (xmmsc_playlist_list conn))]
	 ; Drop all playlists starting with _; those are system hidden
	 ; playlists
	 [playlists (remove
		      (lambda (x) (eq? #\_ (string-ref x 0)))
		      ; Turn the result into a list of strings
		      (result->string-list result))])
    (logger "In show-playlists-list: playlists " playlists nl)
    (set-selection
      (fmt #f (fit maxwidth "   Playlists"))
      playlists
      playlists)
    (show-playlists-help)
    (xmmsc_result_unref result)))

; Create a new, empty, playlist
(define (new-playlist)
  (let* ([playlist-name (my-getString "Enter Playlist Name" "Name: " "")])
    (cond
      ; Make sure we got a good name
      [(and playlist-name (not (string= "" playlist-name)))
       (xmmsc_playlist_create conn playlist-name)]))
  (refresh-current))

; Load a playlist (either the one under the cursor, or the current
; one)
(define (load-playlist object)
  (cond
    ; If we're in a particular playlist, load that one
    [playlist-name
      (my-xmms-wait (xmmsc_playlist_load conn playlist-name))]
    ; Otherwise, find which playlist we're supposed to load
    [else
      (define-external obj2 c-pointer object)
      (let*
	([playlist
	   (list-ref
	     selection-items
	     (getCDKSelectionCurrent object))])
	(my-xmms-wait (xmmsc_playlist_load conn playlist)))])
  (refresh-current))

; Delete a playlist (either the one under the cursor, or the current
; one)
(define (delete-playlist object)
  (let* (
	 ; Get the name of the active playlist
	 [active (result->string (xmmsc_playlist_current_active conn))]
	 [to-remove
	   (cond
	     ; If we're in a particular playlist, delete that one
	     [playlist-name
	       playlist-name]
	     ; Otherwise, find which playlist we're supposed to load
	     [else
	       (define-external obj2 c-pointer object)
	       (list-ref
		 selection-items
		 (getCDKSelectionCurrent object))])])
    (logger "In delete-playlist: " active ", " to-remove nl)
    (cond
      ; Can't remove the current playlist
      [(string= active to-remove)
       (logger "In delete-playlist: can't remove that.\n")
       (popupLabel cdk_screen (make_strings (list "Can't delete the active playlist!")) 1)]
      [else
	(logger "In delete-playlist: removing.\n")
	(my-xmms-wait (xmmsc_playlist_remove conn to-remove))])
    ; No matter what, we *certainly* aren't in any particular playlist
    ; anymore.
    (global-set! 'playlist-name #f)
    (show-playlists)))

; Shuffle the current playlist
(define (playlist-shuffle)
  (logger "In playlist-shuffle.\n")
  (logger "In playlist-shuffle: About to shuffle " (foreign-value "XMMS_ACTIVE_PLAYLIST" c-string) nl)
  (my-xmms-wait (xmmsc_playlist_shuffle conn (foreign-value "XMMS_ACTIVE_PLAYLIST" c-string)))
  (refresh-current))

; Move to the next item in the playlist
(define (playlist-next #!optional no-auto-rate?)
  ; Don't try it if we're already at the bottom of the list
  (logger "In playlist-next, pos check: "
	  (- (length (playlist-get-ids)) 1) ", "
	  (result->int (xmmsc_playlist_current_pos conn #f)))
  (cond
    [(not (=
	    (- (length (playlist-get-ids)) 1)
	    (result->int (xmmsc_playlist_current_pos conn #f))))
     (let*
       ([real-id (playlist-current-id)])
       (cond
	 [(and
	    auto-rating
	    (not no-auto-rate?))
	  (ratings-down real-id)])
       (my-xmms-wait (xmmsc_playlist_set_next_rel conn 1))
       (my-xmms-wait (xmmsc_playback_tickle conn))
       (playlist-id-update-wait real-id)
       (refresh-current))]))

; Move to the previous item in the playlist
(define (playlist-previous)
  ; Don't try it if we're already at the beginning of the list
  (cond
    [(not (= 0 (result->int (xmmsc_playlist_current_pos conn #f))))
     (let*
       ([real-id (playlist-current-id)])
       (my-xmms-wait (xmmsc_playlist_set_next_rel conn -1))
       (my-xmms-wait (xmmsc_playback_tickle conn))
       (cond
	 [auto-rating
	   (playlist-id-update-wait real-id)
	   (ratings-up (playlist-current-id))])
       ; Just in case we're not updated yet
       (playlist-id-update-wait real-id)
       (refresh-current))]))

; Retrieve the current playing id from the server
(define (playlist-current-id)
  (result->int (my-xmms-wait (xmmsc_playback_current_id conn))))

; Loop until the id actually *changes*.  Yes, this is a bit of a
; busy loop, but it shouldn't last long.
(define (playlist-id-update-wait prev-id)
  (logger "In playlist-id-update-wait: prev: " prev-id nl)
  (let id-wait
    ([current-id (playlist-current-id)])
    (logger "In playlist-id-update-wait: current: " current-id nl)
    (cond
      [(and
	 ; If we're not playing, we'd be doing this forever...
	 (= 1 (status-get-playback-status))
	 ; Still the same: keep trying
	 (= current-id prev-id))
       (id-wait (playlist-current-id))])))

; Get the list of ids in the current playlist
(define (playlist-get-ids)
  (result->int-list (xmmsc_playlist_list_entries conn #f)))

; Add all songs in the current view to the playlist
(define (playlist-add-all)
  (for-each
    (lambda (id)
      (my-xmms-wait-toss (xmmsc_playlist_add_id conn (foreign-value "XMMS_ACTIVE_PLAYLIST" c-string) id)))
    selection-items)
  (refresh-current))

(define playlist-others-to-add '())

; Add the given song(s) to a selected playlist.  What we actually do
; here is save the list of songs to add in a global.  Then we call
; the regular playlist lister, but with a special system-state.
; When enter or whatever is hit, we deal with the list.
(define (playlist-other-add object)
  (global-set! 'previous-state system-state)
  (global-set! 'refresh-previous refresh-current)
  (global-set! 'system-state 'playlists-selector)
  (global-set! 'refresh-current (refresh-with show-playlists))
  (logger "In playlist-other-add: " object ", " system-state "\n")
  (handle-choices
    object
    #t
    ; Turn the list of choices into a list of song ids
    (lambda (items)
      (map
	(lambda (x)
	  (list-ref selection-items x))
	items))
    ; For each selection, add it to the current playlist.
    (lambda (id)
      (logger "In playlist-other-add saving " id nl)
      (set! playlist-others-to-add (append (list id) playlist-others-to-add))))
  (show-playlists-list))

(define (playlists-selector-select object)
  (logger "In playlist-selector-select: " object ", " system-state "\n")
  (logger "In playlist-selector-select, the list is: " playlist-others-to-add nl)
  (for-each
    (lambda (songid)
      (define-external obj2 c-pointer object)
      (let*
	([playlist
	   (list-ref
	     selection-items
	     (getCDKSelectionCurrent object))])
	(logger "In playlist-selector-select, adding songid " songid " to playlist " playlist nl)
	(my-xmms-wait-toss (xmmsc_playlist_add_id conn playlist songid))))
    playlist-others-to-add)
  (set! playlist-others-to-add '())
  (revert-state))
