(declare (unit props))

(use posix)

(include "logger.sci")

#>
#include <cdk.h>
#include <xmmsclient/xmmsclient.h>
<#

(define props-song #f)

; Select a song to show properties for
(define (props-display object)
  (let*
    (
     [list-id
       (getCDKSelectionCurrent object)]
     [id
       (list-ref selection-items list-id)])
    (cond
      ; Might be a directory or something
      [id
	(global-set! 'previous-state system-state)
	(global-set! 'refresh-previous refresh-current)
	(set! props-song id)
	(global-set! 'system-state 'props)
	(global-set! 'refresh-current props-refresh)
	(show-props-help)
	(refresh-current)])))

; Display the props list for the selected song.
(define (props-refresh)
  (logger "In props-refresh.\n")
  (let*
    (
     ; Sort props
     [some-props (sort all-props (lambda (x y) (string< (first x) (first y))))]
     ; Get the property values for the song
     [prop-vals
       (and
	 props-song
	 (map
	   (lambda (prop) (conc (first prop) ": " (first (get-mediainfo props-song (list (first prop))))))
	   some-props))]
     )
    (cond
      [prop-vals
	(logger "In props-refresh: prop-vals: " prop-vals nl)
	(set-selection
	  (fmt #f (fit maxwidth "Edit Properties"))
	  some-props
	  prop-vals)])))

(define (props-edit object)
  (let*
    (
     [list-id
       (getCDKSelectionCurrent object)]
     [prop
       (list-ref selection-items list-id)])
    ; Refuse to edit read-only props
    (cond
      [(or
	 (string= (first prop) "id")
	 (string= (first prop) "url")
	 ; These 5 are physical attributes of the song
	 ; file itself, and hence not really
	 ; appropriate targets for changes
	 (string= (first prop) "size")
	 (string= (first prop) "bitrate")
	 (string= (first prop) "channels")
	 (string= (first prop) "sample_format")
	 (string= (first prop) "samplerate")
	 ; Duration is not actually read-only, but it
	 ; is tricky
	 (string= (first prop) "duration")
	 (string= (first prop) "added"))
       (popupLabel cdk_screen (make_strings (list "Not editable.")) 1)]
      [else
	(let*
	  ([prop-val
	     (first (get-mediainfo props-song (list (first prop))))]
	   [ret-string (my-getString "Enter The New Value" (conc prop ": ") prop-val)]
	   )
	  (logger "In edit-props, testing " ret-string " against type " (second prop) nl)
	  ; We only consider int and string props, so if this isn't
	  ; a string prop, we try to coerce int to an integer, and
	  ; barf if that fails.
	  (cond 
	    [(and
	       ; If it's not a string we're looking for (i.e. it's
	       ; an integer)
	       (not (eq? (second prop) (foreign-value "XMMSC_RESULT_VALUE_TYPE_STRING" int)))
	       ; And we didn't get one
	       (or
		 (not (string->number ret-string))
		 (not (integer? (string->number ret-string))))
	       ; And we didn't get a blank string
	       (not (string= ret-string ""))
	       ; And we didn't get no-data
	       (not (string= ret-string no-data)))
	     (logger "In edit-props, testing failed.\n")
	     ; Try, try again.
	     (popupLabel cdk_screen (make_strings (list "Not an integer value.")) 1)]
	    ; If it's an integer, write it as such.
	    [(and
	       ; If it's not a string we're looking for (i.e. it's
	       ; an integer)
	       (not (eq? (second prop) (foreign-value "XMMSC_RESULT_VALUE_TYPE_STRING" int)))
	       ; And we got one
	       (string->number ret-string)
	       (integer? (string->number ret-string))
	       ; And we didn't get a blank string
	       (not (string= ret-string ""))
	       ; And we didn't get no-data
	       (not (string= ret-string no-data)))
	     (xmmsc_medialib_entry_property_set_int_with_source
	       conn				; connection
	       props-song			; song id
	       (find-source props-song (first prop))	; source
	       (first prop)			; key
	       (string->number ret-string))	; value
	     ; Write it out, if we can
	     (write-out-prop props-song (find-source props-song (first prop)) (first prop) (string->number ret-string))
	     ]
	    ; If it's a string write it as such.
	    [(and
	       ; If it's a string we're looking for
	       (eq? (second prop) (foreign-value "XMMSC_RESULT_VALUE_TYPE_STRING" int))
	       ; And we got something
	       ret-string
	       ; And we didn't get no-data
	       (not (string= ret-string no-data)))
	     (logger "In edit-props: Writing to the library: " props-song ", " (find-source props-song (first prop)) ", " (first prop) ", " ret-string nl)
	     (xmmsc_medialib_entry_property_set_str_with_source
	       conn				; connection
	       props-song			; song id
	       (find-source props-song (first prop))	; source
	       (first prop)			; key
	       ret-string)			; value
	     ; Write it out, if we can
	     (write-out-prop props-song (find-source props-song (first prop)) (first prop) ret-string)
	     ]))]))
  (refresh-current))

; This hacky solution writes out properties that seem appropriate by
; making shell calls to the id3v2 and vorbiscomment programs.
;
; Ewwwww.
(define (write-out-prop id source key value)
  (logger "In write-out-prop: " id ", " source ", " key ", " value nl)
  ; If we have auto-writeback turned on.
  (cond
    [prop-writeback
      (let*
	([url (first (get-mediainfo id (list "url")))]
	 ; Get the actual file name
	 [file-name
	   (string-substitute*
	     ; Get the basic file name
	     (string-substitute "^file://" "" url)
	     ; Swap the encoding around
	     (map
	       (lambda (entry)
		 (cons (regexp-escape (cdr entry)) (car entry)))
	       xmms-file-name-encoding)
	     #f)]
	 ; Convert the actual file name to something that won't
	 ; break up when we pass it to the shell
	 [file-name-shell
	   (conc "\"" (string-substitute "\"" "\\\"" file-name #f) "\"")]
	 ; First see what type of file we have.
	 [mp3? (string-match "file://.*\\.mp3" url)]
	 [ogg? (string-match "file://.*\\.ogg" url)]
	 [ogg-props-tmp (conc "/tmp/cluck.ogg.props." (current-process-id))]
	 [ogg-file-tmp (conc "/tmp/cluck.ogg." (current-process-id))]
	 [handle-mp3 (lambda (key arg)
		       (cond
			 [(string= key "comment")
			  ; Special case for comments: clean all the
			  ; wierd bits out.
			  (process-wait (process-run (conc "id3v2 -2 -c 'ID3v1 Comment::XXX' " file-name-shell " >/dev/null")))
			  (process-wait (process-run (conc "id3v2 -2 -c '' " file-name-shell " >/dev/null")))])
		       (logger "About to run id3v2 on " arg ", " value ", " file-name-shell nl)
		       (process-wait (process-run "id3v2" (list "-2" (->string arg) (->string value) file-name))))])
	(cond
	  [mp3?
	    ; Convert and delete id3v1 tags
	    (process-wait (process-run (conc "id3v2 -C " file-name-shell " >/dev/null")))
	    (process-wait (process-run (conc "id3v2 -s " file-name-shell " >/dev/null")))
	    ; If it's an MP3 type, just set any keys that match the tags
	    ; id3v2 knows about on the command line.
	    (cond
	      [(or
		 (string= key "genre")
		 (string= key "album")
		 (string= key "comment")
		 (string= key "artist"))
	       (handle-mp3 key (conc "--" key))]
	      [(string= key "date")
	       (handle-mp3 key "--year")]
	      [(string= key "title")
	       (handle-mp3 key "--song")]
	      [(string= key "tracknr")
	       (handle-mp3 key "--track")])
	    ]
	  [ogg?
	    ; If it's an Ogg file, set any of the obvious one's, and
	    ; anything that was set by plugin/vorbis
	    ;
	    ; But first, save the current bits
	    (process-wait (process-run (conc "vorbiscomment -l " file-name-shell " >" ogg-props-tmp)))
	    (let*
	      ([data (read-all ogg-props-tmp)]
	       ; Set a pair (xmms2-name vorbis-name) based on the prop
	       ; name
	       [name-pair
		 (cond
		   [(or
		      (string= key "genre")
		      (string= key "date")
		      (string= key "album")
		      (string= key "comment")
		      (string= key "artist")
		      (string= key "title"))
		    (list key (string-upcase key))]
		   [(string= key "tracknr")
		    (list key "TRACKNUMBER")]
		   ; Preserve anything else that was originally set by
		   ; plugin/vorbis
		   [(string= source "plugin/vorbis")
		    (list key (string-upcase key))]
		   [else #f])]
	       [out-port (open-output-file ogg-props-tmp)])
	      (logger "Vorbis data in: " data nl)
	      (logger "name-pair: " name-pair nl)
	      ; If name-pair was actually set, do something with it
	      (cond
		[name-pair
		  ; Write the new version to the tmp file
		  (write-string
		    (conc
		      (second name-pair)
		      "="
		      value
		      "\n"
		      (string-intersperse
			; Drop the cruuent value, if any
			(remove
			  (lambda (x)
			    (logger "String-prefix test: " (conc (second name-pair) "=") ", " x nl)
			    (string-prefix? (conc (second name-pair) "=") x))
			  (string-split data "\n"))
			"\n"))
		    #f
		    out-port)
		  (close-output-port out-port)
		  (cond
		    [(>=
		       (length
			 (string-split (read-all ogg-props-tmp) "\n"))
		       (length (string-split data "\n")))
		     (logger "About to write vorbiscomment -w -c " ogg-props-tmp ", " file-name " -- " (read-all ogg-props-tmp) nl)
		     ; Write to a tmp file because vorbiscomment has
		     ; zero-length-file sort of failure modes :P
		     (process-wait (process-run (conc "vorbiscomment -w -c " ogg-props-tmp " " file-name-shell " " ogg-file-tmp)))
		     ; See if we got a good tmp file out of that
		     (cond
		       [(and
			  (file-exists? ogg-file-tmp)
			  (regular-file? ogg-file-tmp)
			  ; New file should be within 20% of old file size
			  (>
			    (inexact->exact (round (* 1.2 (file-size ogg-file-tmp))))
			    (file-size file-name)))
			(rename-file ogg-file-tmp file-name)]
		       [else
			 (popupLabel cdk_screen (make_strings (list (conc "Write of prop " key " as ogg comment."))) 1)
			 ])
		     ]
		    [else
		      (logger "Bad number of lines for ogg writeback: " (read-all ogg-props-tmp) nl)])]))
	    (delete-file* ogg-props-tmp)
	    (delete-file* ogg-file-tmp)]))]))



; Add a prop to a song.
(define (add-prop object)
  (logger "In add-prop.\n")
  (foreign-code "curs_set( 1 );" )
  (define-external obj2 c-pointer object)
  (let*
    (
     ; widget to get the property name from the user
     [name-widget
       (newCDKEntry
	 cdk_screen
	 5		; xpos
	 5		; ypos
	 ""		; title
	 "Property Name: " 		; label
	 (foreign-value "A_NORMAL" unsigned-int)	; fieldAttribute
	 (foreign-value "' '" unsigned-int)		; fillerCharacter
	 (foreign-value "vMIXED" unsigned-int)		; displayType
	 30		; field width
	 0		; minimumLength
	 4096		; maximumLength
	 1		; box
	 0		; shadow
	 )]
     ; widget to get the property type from the user
     [scroller
       (newCDKScroll
	 cdk_screen					; CDKSCREEN *cdkscreen,
	 5						; int xpos,
	 7						; int ypos,
	 (foreign-value "RIGHT" unsigned-int)		; int spos,
	 10						; int height,
	 30						; int width,
	 "Property Type"				; char *title,
	 (make_strings (list "number" "string"))	; char **itemList,
	 2						; int items,
	 0						; boolean numbers,
	 (foreign-value "A_REVERSE" unsigned-int)	; chtype highlight,
	 1						; boolean box,
	 0)						; boolean shadow);
       ])

    (drawCDKEntry name-widget 1)
    (drawCDKScroll scroller 1)

    (let*
      ([prop-name (activateCDKEntry name-widget #f)]
       [prop-type-num (activateCDKScroll scroller #f)])
      (logger "In add-prop: " prop-name ", " prop-type-num nl)
      ; If the user exited either, don't try anything
      (cond
	[(and prop-name prop-type-num)
	 (let*
	   ([prop-type (list-ref
			 (list
			   (foreign-value "XMMSC_RESULT_VALUE_TYPE_UINT32" int)
			   (foreign-value "XMMSC_RESULT_VALUE_TYPE_STRING" int))
			 prop-type-num)])
	   (logger "In add-prop: " prop-name ", " prop-type-num ", " prop-type nl)
	   (logger "In add-prop: song id: " props-song nl)
	   (cond
	     [(= prop-type-num 0)
	      (my-xmms-wait
		(xmmsc_medialib_entry_property_set_int
		  conn			; connection
		  props-song		; song id
		  prop-name		; key
		  1))]			; value
	     [(= prop-type-num 1)
	      (my-xmms-wait
		(xmmsc_medialib_entry_property_set_str
		  conn			; connection
		  props-song		; song id
		  prop-name		; key
		  ""))])		; value
	   (global-set!
	     'all-props
	     (cons (list prop-name prop-type) all-props))
	   (destroyCDKEntry name-widget)
	   (destroyCDKScroll scroller))]))))
