(declare (unit ratings))

#>
#include <xmmsclient/xmmsclient.h>
<#

(include "logger.sci")

(define ratings-song #f)

(define all-ratings
  (list
    "100 = Awful"
    "200 = Bad"
    "300 = Neutral"
    "400 = Good"
    "500 = Superb"))

(define (ratings-userid-check)
  (cond
    [(not userid)
     (let*
       ([new-userid (my-getString "Enter A Ratings Username:" "Name: " "")])
       (cond
	 [(and
	    new-userid
	    (not (string= "" new-userid)))
	  (global-set! 'userid new-userid)]
	 [else
	   (popupLabel cdk_screen (make_strings (list "Hmmm, that didn't seem to work.")) 1)
	   (ratings-userid-check)]))]))

; Select a rating for this object using a popup list
(define (ratings-select object)
  (logger "In ratings-select.\n")
  (ratings-userid-check)
  (let*
    (
     [list-id
       (getCDKSelectionCurrent object)]
     [id
       (list-ref selection-items list-id)])
    (global-set! 'previous-state system-state)
    (global-set! 'refresh-previous refresh-current)
    (set! ratings-song id)
    (global-set! 'system-state 'ratings)
    (global-set! 'refresh-current ratings-refresh)
    (refresh-current)))

(define (ratings-refresh)
  (logger "In ratings-refresh: ratings-song: " ratings-song nl)
  (ratings-userid-check)
  (let*
    ([current-rating (first (get-mediainfo ratings-song (list (conc "rating." userid))))]
     ; Get an at-most singleton list of which rating this song is
     ; using
     [used
       (cond
	 [(string= no-data current-rating)
	  ""]
	 [else (list-ref all-ratings (- (min 5 (max 1 (quotient (string->number current-rating) 100))) 1))])]
     ; Highlight the used rating, if any
     [our-ratings
       (map
	 (lambda (line)
	   (cond
	     [(string= used line)
	      (conc good-highlight line)]
	     [else line]))
	 all-ratings)])
    (logger "In ratings-refresh: used: " used ", our-ratings: " our-ratings nl)
    (show-ratings-help)
    ; The list here is the current columns, in order, followed
    ; by the rest.
    (set-selection
      (fmt #f (fit maxwidth "Set Rating"))
      ; The scheme selection list (i.e. the raw data)
      (list 100 200 300 400 500)
      ; The actual text to display.
      our-ratings)))

; Set the previously-chosen song to the selected rating
(define (ratings-set object)
  (let*
    (
     [list-id
       (getCDKSelectionCurrent object)]
     [rating
       (list-ref selection-items list-id)])
    (xmmsc_medialib_entry_property_set_int conn ratings-song (conc "rating." userid) rating))
  (revert-state))

; Toggle whether we are automatically rating songs or not
(define (ratings-auto)
  (logger "In ratings-auto.\n")
  ; Turn it on
  (cond
    [(not auto-rating)
     (cond
       ; Confirm the rate with the user
       [(not auto-rating-change)
	(let get-rate
	  ([new-rate (my-getString "Enter The Rate Of Auto-Rating Change" "Rate: " "20")])
	  (cond
	    [(not (integer? (string->number new-rate)))
	     ; Try again
	     (popupLabel cdk_screen (make_strings (list "Must be an integer.")) 1)
	     (get-rate (my-getString "Enter The Rate Of Auto-Rating Change" "Rate: " "20"))]
	    [else
	      (global-set! 'auto-rating-change (string->number new-rate))]))])
     (global-set! 'auto-rating #t)]
    [auto-rating
      (global-set! 'auto-rating #f)]))

; Called for downgrading sorts of rating events
(define (ratings-down id)
  (logger "In ratings-down: id: " id nl)
  (cond
    [auto-rating
      (let*
	([current-rating-raw (first (get-mediainfo id (list (conc "rating." userid))))]
	 [current-rating
	   (cond
	     [(string= no-data current-rating-raw)
	      300]
	     [else (string->number current-rating-raw)])])
	(xmmsc_medialib_entry_property_set_int
	  conn
	  id
	  (conc "rating." userid)
	  (- current-rating auto-rating-change)))]))

; Called for upgrading sorts of rating events
(define (ratings-up id)
  (logger "In ratings-up: id: " id nl)
  (cond
    [auto-rating
      (let*
	([current-rating-raw (first (get-mediainfo id (list (conc "rating." userid))))]
	 [current-rating
	   (cond
	     [(string= no-data current-rating-raw)
	      300]
	     [else (string->number current-rating-raw)])])
	(xmmsc_medialib_entry_property_set_int
	  conn
	  id
	  (conc "rating." userid)
	  (+ current-rating auto-rating-change)))]))

; Toggle auto-skipping 
(define (ratings-auto-skip-toggle)
  ; Turn it on
  (cond
    [(not auto-skip)
     ; Confirm the limit with the user every time
     (let get-rate
       ([new-rate (my-getString "Enter The Limit Below Which To Skip" "Limit: " "210")])
       (cond
	 [(not (integer? (string->number new-rate)))
	  ; Try again
	  (popupLabel cdk_screen (make_strings (list "Must be an integer.")) 1)
	  (get-rate (my-getString "Enter The Limit Below Which To Skip" "Limit: " "210"))]
	 [else
	   (global-set! 'auto-skip-limit (string->number new-rate))]))
     (global-set! 'auto-skip #t)]
    [auto-skip
      (global-set! 'auto-skip #f)])
  (refresh-current))

; Auto-skip the current song; or not.
(define (ratings-auto-skip id)
  (let*
    ([current-rating-raw (first (get-mediainfo id (list (conc "rating." userid))))]
     [current-rating
       (cond
	 [(string= no-data current-rating-raw)
	  300]
	 [else (string->number current-rating-raw)])])
    (cond
      [(and auto-skip (< current-rating auto-skip-limit))
       (playlist-next #t)])))
